
-----------------------------------------------------------------------------------------
--
-- Copyright (c) 2009-2012 ProactiveRA
--
-- This script is free software; you can redistribute it and/or modify it under
-- the terms of the GNU Lesser General Public License as published by the Free
-- Software Foundation; either version 3 of the License, or (at your option)
-- any later version.
--
-- This script is distributed in the hope that it will be useful, but WITHOUT
-- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-- FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
-- details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-----------------------------------------------------------------------------------------



DROP PROCEDURE P_KPI_UPDATE_HISTORY;

CREATE OR REPLACE PROCEDURE P_KPI_UPDATE_HISTORY
IS
  CURSOR C_LAST_YEAR IS 
	SELECT A.KPI_ID, A.CALCULATED_VALUE, A.CALCULATED_DATE
	FROM KPI_CALCULATED_VALUES_HISTORY A
	WHERE KPI_ID IN
	(SELECT KPI_ID
	 FROM REF_KPI_DEFINITION
	 WHERE ACTIVE=1
	)
	AND ABS(A.CALCULATED_DATE-(TRUNC(SYSDATE, 'DD')-365))=(
		SELECT MIN(ABS(B.CALCULATED_DATE-(TRUNC(SYSDATE, 'DD')-365))) 
		FROM KPI_CALCULATED_VALUES_HISTORY B
		WHERE B.KPI_ID=A.KPI_ID
		AND B.CALCULATED_DATE>=(TRUNC(SYSDATE, 'DD')-365-6)
		AND B.CALCULATED_DATE<(TRUNC(SYSDATE, 'DD')-365+6)
	)
	AND A.CALCULATED_DATE>=(TRUNC(SYSDATE, 'DD')-365-6)
	AND A.CALCULATED_DATE<(TRUNC(SYSDATE, 'DD')-365+6);

  CURSOR C_LAST_MONTH IS
	SELECT A.KPI_ID, A.CALCULATED_VALUE, A.CALCULATED_DATE
	FROM KPI_CALCULATED_VALUES_HISTORY A
	WHERE KPI_ID IN
	(SELECT KPI_ID
	 FROM REF_KPI_DEFINITION
	 WHERE ACTIVE=1
	)
	AND ABS(A.CALCULATED_DATE-(ADD_MONTHS(TRUNC(SYSDATE, 'DD'),-1)))=(
		SELECT MIN(ABS(B.CALCULATED_DATE-(ADD_MONTHS(TRUNC(SYSDATE, 'DD'),-1))))
		FROM KPI_CALCULATED_VALUES_HISTORY B
		WHERE B.KPI_ID=A.KPI_ID
		AND B.CALCULATED_DATE>=(ADD_MONTHS(TRUNC(SYSDATE, 'DD'), -1)-8)
		AND B.CALCULATED_DATE<(ADD_MONTHS(TRUNC(SYSDATE, 'DD'), -1)+8)
	)
	AND A.CALCULATED_DATE>=(ADD_MONTHS(TRUNC(SYSDATE, 'DD'),-1)-8)
	AND A.CALCULATED_DATE<(ADD_MONTHS(TRUNC(SYSDATE, 'DD'),-1)+8);

  CURSOR C_LAST_WEEK IS 
	SELECT A.KPI_ID, A.CALCULATED_VALUE, A.CALCULATED_DATE
	FROM KPI_CALCULATED_VALUES_HISTORY A
	WHERE KPI_ID IN
	(SELECT KPI_ID
	 FROM REF_KPI_DEFINITION
	 WHERE ACTIVE=1
	)
	AND ABS(A.CALCULATED_DATE-(TRUNC(SYSDATE, 'DD')-7))=(
		SELECT MIN(ABS(B.CALCULATED_DATE-(TRUNC(SYSDATE, 'DD')-7))) 
		FROM KPI_CALCULATED_VALUES_HISTORY B
		WHERE B.KPI_ID=A.KPI_ID
		AND B.CALCULATED_DATE>=(TRUNC(SYSDATE, 'DD')-7-5)
		AND B.CALCULATED_DATE<(TRUNC(SYSDATE, 'DD')-7+5)
	)
	AND A.CALCULATED_DATE>=(TRUNC(SYSDATE, 'DD')-7-5)
	AND A.CALCULATED_DATE<(TRUNC(SYSDATE, 'DD')-7+5);

  CURSOR C_YESTERDAY IS
	SELECT A.KPI_ID, A.CALCULATED_VALUE, A.CALCULATED_DATE
	FROM KPI_CALCULATED_VALUES_HISTORY A
	WHERE KPI_ID IN
	(SELECT KPI_ID
	 FROM REF_KPI_DEFINITION
	 WHERE ACTIVE=1
	)
	AND ABS(A.CALCULATED_DATE-(TRUNC(SYSDATE, 'DD')-1))=(
		SELECT MIN(ABS(B.CALCULATED_DATE-(TRUNC(SYSDATE, 'DD')-1)))
		FROM KPI_CALCULATED_VALUES_HISTORY B
		WHERE B.KPI_ID=A.KPI_ID
		AND B.CALCULATED_DATE>=(TRUNC(SYSDATE, 'DD') -1)
	)
	AND A.CALCULATED_DATE>=(TRUNC(SYSDATE, 'DD')-1);

BEGIN

  -- RESET HISTORY FIRST BEFORE RECALCULATION
  UPDATE KPI_CALCULATED_VALUES
  SET 	LAST_YEAR_VALUE=NULL,
	LAST_MONTH_VALUE=NULL,
	LAST_WEEK_VALUE=NULL,
	YESTERDAY_VALUE=NULL;

  -- LAST YEAR's VALUES
  FOR C1 IN C_LAST_YEAR LOOP 
	UPDATE KPI_CALCULATED_VALUES
	SET LAST_YEAR_VALUE=C1.CALCULATED_VALUE
	WHERE KPI_ID=C1.KPI_ID;
  END LOOP;

  -- LAST MONTH's VALUES
  FOR C2 IN C_LAST_MONTH LOOP 
	UPDATE KPI_CALCULATED_VALUES
	SET LAST_MONTH_VALUE=C2.CALCULATED_VALUE
	WHERE KPI_ID=C2.KPI_ID;
  END LOOP;

  -- LAST WEEK's VALUES
  FOR C3 IN C_LAST_WEEK LOOP 
	UPDATE KPI_CALCULATED_VALUES
	SET LAST_WEEK_VALUE=C3.CALCULATED_VALUE
	WHERE KPI_ID=C3.KPI_ID;
  END LOOP;

  -- YESTERDAY's VALUES
  FOR C4 IN C_YESTERDAY LOOP 
	UPDATE KPI_CALCULATED_VALUES
	SET YESTERDAY_VALUE=C4.CALCULATED_VALUE
	WHERE KPI_ID=C4.KPI_ID;
  END LOOP;

END;
/



